/*
 *    ks2tex.c  : a ks-to-tex converter
 *    Copyright 1992, Uhhyung Choi
 *
 *    Revision history:
 *
 *    Created By uhhyung@daiduk.kaist.ac.kr on 10-Oct-1991.
 *    Updated to support a single character block on 29-May-1992.
 *    Added Hanja part by zraun01@hpserv.zdv.uni-tuebingen.de on 04-Jul-1993.
 *    Changed Mule part of Hanja (thank handa@etl.go.jp) on 23-Jul-1993.
 *          
 * 
 *    Comments:
 *
 *    String XCHARS is defined to allow extra space \hX for specific characters
 *    which comes just before a Hangul character.
 *    Macro \hS is for similar cases execpt that it's for English alphabets.
 *
 *    For use Hanja you need the watanabe-jfonts and Mule.
 *    In Mule, set *euc-korea* for file-coding-system of a text
 *    which contains Hangul and Hanja.
 *
 *    Feel free to customize the values defined in "hhead.tex".
 *
 *    This is freely distributable under GNU public license.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include "table.h"

#define XCHARS	"$()"

int line;

void usage()
{
  fputs("Usage: htex [Input] [Output]\n",stderr);
}

int Fgetc(in)
FILE *in;
{
  int c;
  c = fgetc(in);
  if(c=='\n') line++;
  return c;
}

void main(argc,argv)
int argc;
char **argv;
{
  FILE *in,*out;
  extern unsigned char htbl[][4];
  int opt, ch=0, last=0, font=0, mule=0;
  while((opt=getopt(argc,argv,"h"))!=-1) {
    usage();
    exit(0);
  }
  in = argc > 1 ? fopen(argv[1],"r") : stdin;
  out = argc > 2 ? fopen(argv[2],"w") : stdout;
  if (in==NULL){
    printf("%s : not found\n",argv[1]);
    exit(0);
  }
  if (argc==1) {
    puts("% htex : KS Hangul and JIS Hanja to ASCII TeX code converter");
    puts("% Press 'ctrl-d' to exit\n%");
  }
  line=1;
  ch = Fgetc(in);
  while(!feof(in)) {
    if (ch>160) {		/* Hangul codes */
      int code,code2;
      code = 94*(ch-176);
      code2 = Fgetc(in);
      if(code2<=160){
	fprintf(stderr,"bad char at line %d\n",line);
	ch = Fgetc(in);
	continue;
      }
      code +=(code2-161);
      if(ch==0xa4) {
	code = 2350+code2-161;
      } else if(code<0||code>2349) {
	fprintf(stderr,"bad Hangul in line %d\n",line);
	ch = Fgetc(in);
	continue;
      }
      fputc('{',out);
      if (isalpha(last)) fprintf(out,"\\hS");
      if (strchr(XCHARS,last)) fprintf(out, "\\hX");

      font = htbl[code][0];
      if (font==1) fprintf(out,"\\hA");
      else fprintf(out,"\\hB");
      if (htbl[code][3]) {
	fprintf(out,"\\hC%d\\hC%d\\hC%d",
		htbl[code][1],htbl[code][2],
		htbl[code][3]);
      } else {
	if (htbl[code][2]!=32)
	  fprintf(out,"\\hC%d\\hC%d",
		  htbl[code][1],htbl[code][2]);
	else
	  fprintf(out,"\\hC%d\\kern 1em", htbl[code][1]);	
      }
      last = ch;
      ch = Fgetc(in);
      if (ch>160 || (ch!=27 && mule==1) || (ch==27 && mule==0)) fprintf(out,"\\hP");
      fputc('}',out);
    } else {			/* not Hangul codes */
      if (ch==27) {
	ch = Fgetc(in) ;
	if (ch==36) {		/* ESC$(A or ESC$(B --> ISO2022 esc seq */
	  mule = 1;
	  ch = Fgetc(in);
	  if (ch==40) ch = Fgetc(in);
	  if (ch==65 || ch==66) ch = Fgetc(in);
				/* A for JISX0208, B for GB2312 designation */
	  else {
	    fprintf(stderr,"unknown Hanja-coding-system at line %d\n",line);
	    ch = Fgetc(in);
	  }
	}
	else if (ch==40) {	/* ESC(B --> designate ASCII */
	  mule = 0;
	  ch = Fgetc(in);
	  if (ch==66) ch = Fgetc(in);
	  else {
	    fprintf(stderr,"corrupt Hanja-coding-system at line %d\n",line);
	    ch = Fgetc(in);
	  }
	}
      }
      else if (mule==1) {	/* the pair of Hanja code is to come */
	int code, code2;
	code = ( ch - 48 ) * 94;
	if (ch>=80) code += 64;
	ch = Fgetc(in);
	code += (ch - 33);
	font = (code / 256) + 65;
	code2 = code % 256;
	if (code<0 || code>6459 || (code>2964 && code<3072)) {
	  fprintf(stderr,"bad Hanja at line %d\n",line);
	  ch = Fgetc(in);
	  continue;
	}
	fputc('{',out);
	if (isalpha(last)) fprintf(out,"\\hS");
	if (strchr(XCHARS,last)) fprintf(out,"\\hX");

	fprintf(out,"\\hj\\j%c\\hC%d",font,code2);
	last = 200;
	ch = Fgetc(in);
	if (ch != 27) fprintf(out,"\\hP");
	fputc('}',out);
      }
      else {			/* plain ASCII codes */
	putc(ch,out);
	last = ch;
	ch = Fgetc(in);
      }
    }
  }
  fclose(in);
  fclose(out);
}
