/*
 *    ks2tex.c  : a ks-to-tex converter
 *
 *    Revision history:
 *
 *    Created By uhhyung@daiduk.kaist.ac.kr on 10-Oct-1991.
 *	minor modification by yunju@camars.kaist.ac.kr on 16-Oct-1991     
 *    Hanja support by zraun01@hpserv.zdv.uni-tuebingen.de on 06-May-1993
 *	added the hangul-jaso code of uhhyung@daiduk.kaist.ac.kr on 29-May-1992
 * 
 *    Comments:
 *
 *    String XCHARS is defined to allow extra space \hX for specific characters
 *    which comes just before a Hangul character.
 *    Macro \hS is for similar cases execpt that it's for English alphabets.
 *
 *    Feel free to customize the values defined in "hfont.tex".
 *	
 *
 *    This is freely distributable under GPL license term.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include "table.h"

#define XCHARS	"$()"

int line;

int index(int);
int hanjaa(int, int);
int hanjab(int, int);
int hanjac(int, int);
int hanjad(int, int);
int hanjae(int, int);

void usage()
{
  fputs("Usage: htex [Input] [Output]\n",stderr);
}

int Fgetc(in)
FILE *in;
{
  int c;
  c = fgetc(in);
  if(c=='\n') line++;
  return c;
}

void main(argc,argv)
int argc;
char **argv;
{
  FILE *in,*out;
  extern unsigned char htbl[][4];
  int opt, ch=0, last=0, font=0;
  while((opt=getopt(argc,argv,"h"))!=-1) {
    usage();
    exit(0);
  }
  in = argc > 1 ? fopen(argv[1],"r") : stdin;
  out = argc > 2 ? fopen(argv[2],"w") : stdout;
  if(in==NULL){
    printf("%s : not found\n",argv[1]);
    exit(0);
  }
  if(out==NULL){
    printf("%s : not found\n",argv[2]);
    exit(0);
  }
  if (argc==1) {
    puts("% htex : KS hangul to ASCII TeX code converter");
    puts("% Press 'ctrl-d' \n%");
  }
  line=1;
  ch = Fgetc(in);
  while(!feof(in)) {
    if ( ch==92 ) {			/* hanja begins with backslash */
      ch = Fgetc(in);
      if (ch>160) {			/* That's hanja */
	int code, code2, jdigit;
	unsigned char spec;
	code = 94*(ch-176);
	code2 = Fgetc(in);
	if(code2<=160){
	  fprintf(stderr,"bad char2 at line %d\n",line);
	  ch = Fgetc(in);
	  continue;
	}
	code +=(code2-161);
	if(code<0||code>2349){		/* pseudo hanja */
	  fprintf(stderr,"bad hanja at line %d\n",line);
	  ch = Fgetc(in);
	  continue;
	}
	code = index(code);
	fputc('{',out);
	if (isalpha(last)) fprintf(out,"\\hS");
	if (strchr(XCHARS,last)) fprintf(out, "\\hX");
	last = ch;
	ch = Fgetc(in);
	if (ch != 91 ) {
	  fprintf(stderr,"hanja[#] at line %d doesn't begin with \"[\"\n",line);
	  fputc('}',out);
	  ch = Fgetc(in);
	  continue;
	}
	ch = Fgetc(in);
	if (!isdigit(ch)) {
	  fprintf(stderr,"hanja[#] at line %d must have digit(s)\n",line);
	  fputc('}',out);
	  continue;
	}
	jdigit = ch - 48;		/* the next one or two digit(s)	*/
	ch = Fgetc(in);
	if (isdigit(ch)) {		/* two digit hanja-number	*/
	  jdigit = jdigit*10 + ch - 48;
	  ch = Fgetc(in);
	  if (ch != 93) {
	    fprintf(stderr,"hanja[#] at line %d doesn't end with \"]\"\n",line);
	    fputc('}',out);
	    continue;
	  }
	}
	else {
	  if (ch != 93) {
	    fprintf(stderr,"hanja[#] at line %d doesn't end with \"]\"\n",line);
	    fputc('}',out);
	    continue;
	  }
	}
	if (jdigit > 92)
	  fprintf(stderr,"hanja[#] at line %d exceeds the boundary\n",line);
	jdigit--;
	if (code != 1000) {
	  if (code < 100) code = hanjaa(code,jdigit);
	  else if (code >= 100 && code < 200) code = hanjab(code-100,jdigit);
	  else if (code >= 200 && code < 300) code = hanjac(code-200,jdigit);
	  else if (code >= 300 && code < 400) code = hanjad(code-300,jdigit);
	  else code = hanjae(code-400,jdigit);
	  if (code) {
	    code2 = code%1000;
	    spec = code/1000 + 64;
	    fprintf(out,"\\hj\\j%c\\hC%d", spec, code2);
	  }
	  else {
	    fprintf(stderr,"hanja[#] at line %d exceeds the boundary\n",line);
	    fputc('}',out);
	    ch = Fgetc(in);
	    continue;
	  }
	}
	else fprintf(stderr,"hanja at line %d is not present!\n", line);
	ch = Fgetc(in);
	if (ch > 160) fprintf(out, "\\hP");	/* the next is hangul code */
	else if ( ch==92 ) {
	  code = Fgetc(in);
	  if (code > 160) {			/* the next is hanja code  */
	    fprintf(out, "\\hP");
	    ungetc(code, in);
	  }
	}
	fputc('}',out);
      } else {				/* if not ch>160	*/
	fprintf(out, "\\%c", ch);
	ch = Fgetc(in);
	continue;
      }
    } else if ( ch>160 ) {		/* if !(ch==92) && (hangul char) */
      int code,code2;
      code = 94*(ch-176);
      code2 = Fgetc(in);
      if(code2<=160){
	fprintf(stderr,"bad hangul at line %d\n",line);
	ch = Fgetc(in);
	continue;
      }
      code +=(code2-161);
      if(ch==0xa4) {
	code = 2350+code2-161;
	} else if(code<0||code>2349) {
	  fprintf(stderr,"bad hangul at line %d\n",line);
	  ch = Fgetc(in);
	  continue;
      }
      fputc('{',out);
      if (isalpha(last)) fprintf(out,"\\hS");
      if (strchr(XCHARS,last)) fprintf(out, "\\hX");
      font = htbl[code][0];
      if (font==1) fprintf(out,"\\hA");
      else fprintf(out,"\\hB");
      if (htbl[code][3]) {
	fprintf(out,"\\hC%d\\hC%d\\hC%d",
		htbl[code][1],htbl[code][2],
		htbl[code][3]);
      } else {
	if (htbl[code][2]!=32)
	  fprintf(out,"\\hC%d\\hC%d",
		  htbl[code][1],htbl[code][2]);
	else fprintf(out,"\\hC%d\\kern 1em", htbl[code][1]);
      }
      last = ch;
      ch = Fgetc(in);
      if (ch>160) fprintf(out,"\\hP");
      fputc('}',out);
    } else {
      putc(ch,out);
      last = ch;
      ch = Fgetc(in);
    }
  }
  fclose(in);
  fclose(out);
}
